<?php
// ********************************************************************************************
// Title          udde Picture Frame Information System (uddePF)
// Description    Picture Frame Information System for Joomla 1.5 and Joomla 1.6
// Author          2011 Stephan Slabihoud
// License        This is free software and you may redistribute it under the GPL.
//                uddePF comes with absolutely no warranty.
//                Use at your own risk. For details, see the license at
//                http://www.gnu.org/licenses/gpl.txt
//                Other licenses can be found in LICENSES folder.
//                Redistributing this file is only allowed when keeping the header unchanged.
//
// It uses parts of
//
// InfoFrame (image generator for digital picture frames)
// Copyright (C) 2009  Tobias Kolb
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// ********************************************************************************************

require_once 'imagettftextboxopt.php';

class uddePFtools {

	function utf8_to_nce($utf = '') {
		if ($utf == '') return($utf);

		$max_count = 5; // flag-bits in $max_mark ( 1111 1000 == 5 times 1)
		$max_mark = 248; // marker for a (theoretical ;-)) 5-byte-char and mask for a 4-byte-char;

		$html = '';
		for ($str_pos = 0; $str_pos < strlen($utf); $str_pos++) {
			$old_chr = $utf{$str_pos};
			$old_val = ord( $utf{$str_pos} );
			$new_val = 0;
			$utf8_marker = 0;

			// skip non-utf-8-chars
			if( $old_val > 127 ) {
				$mark = $max_mark;
				for($byte_ctr = $max_count; $byte_ctr > 2; $byte_ctr--) {
					// actual byte is utf-8-marker?
					if( ( $old_val & $mark  ) == ( ($mark << 1) & 255 ) ) {
						$utf8_marker = $byte_ctr - 1;
						break;
					}
					$mark = ($mark << 1) & 255;
				}
			}

			// marker found: collect following bytes
			if ($utf8_marker > 1 and isset( $utf{$str_pos + 1} ) ) {
				$str_off = 0;
				$new_val = $old_val & (127 >> $utf8_marker);
				for ($byte_ctr = $utf8_marker; $byte_ctr > 1; $byte_ctr--) {
					// check if following chars are UTF8 additional data blocks
					// UTF8 and ord() > 127
					if( (ord($utf{$str_pos + 1}) & 192) == 128 ) {
						$new_val = $new_val << 6;
						$str_off++;
						// no need for Addition, bitwise OR is sufficient
						// 63: more UTF8-bytes; 0011 1111
						$new_val = $new_val | ( ord( $utf{$str_pos + $str_off} ) & 63 );
					}
					// no UTF8, but ord() > 127
					// nevertheless convert first char to NCE
					else {
						$new_val = $old_val;
					}
				}

				// build NCE-Code
				$html .= '&#'.$new_val.';';
				// Skip additional UTF-8-Bytes
				$str_pos = $str_pos + $str_off;
			} else {
				$html .= chr($old_val);
				$new_val = $old_val;
			}
		}
		return($html);
	}

	function getBiggestFontsizeForText($fontfile, $maxwidth, $text) {
		$size = 132; // biggest possible size is 128
		do {
			$size = $size - 4;
			$bbox = imagettfbbox($size, 0, $fontfile, $text);
			$textWidth = $bbox[2] - $bbox[0];
		} while (($textWidth >= $maxwidth) && ($size >= 12));
		return $size;
	}

	
	public function htmlColorToRgb($htmlcolor) {
		// returns an array of R, G, B values
		return sscanf($htmlcolor, '#%2x%2x%2x');
	}

	public function drawGlassButton($dst_image, $color, $centered_xpos, $centered_ypos) {
		$button_width = 24;
		$button_height = 15;
		// first draw a solid colored circle
		imagefilledellipse($dst_image, $centered_xpos, $centered_ypos, $button_width, $button_height, $color);
		// then draw transparent PNG with glass effect over the solid circle
		$imagefile = dirname(__FILE__).'/../resources/icons/buttonglass.png';
		$icon = ImageCreateFromPNG ( $imagefile );
		imagecopyresampled($dst_image, $icon, $centered_xpos-($button_width/2), $centered_ypos-($button_height/2), 0, 0, $button_width, $button_height, imagesx($icon), imagesy($icon));
	}

	public function loadLanguage( $plugin="", $lang="english" ) {
		$pathtoadmin = dirname(__FILE__).'/..';
		$usedlanguage="";
		if ($plugin) {
			$plugin = "/plugins/".$plugin;
		}
		if (file_exists($pathtoadmin.$plugin.'/language/'.$lang.'.php')) {
			include_once($pathtoadmin.$plugin.'/language/'.$lang.'.php');
			$usedlanguage=$plugin.'/language/'.$lang.'.php';
		} elseif (file_exists($pathtoadmin.$plugin.'/language/english.php')) {
			include_once($pathtoadmin.$plugin.'/language/english.php');
			$usedlanguage=$plugin.'/language/english.php';
		}
		return $usedlanguage;
	}

	public function loadValue($connection, $prefix, $key, $value="") {
		$query = "SELECT value FROM ".$prefix."uddepf_config"." WHERE plugin = 'PFSystem_PERS' AND varname = '".mysql_real_escape_string($key)."'";
		$result = mysql_query($query, $connection) or die('Error: '.mysql_error());
		if (mysql_num_rows($result)==0) {
			$query = "INSERT INTO `".$prefix."uddepf_config"."` (`plugin`, `varname`, `value`) VALUES ('PFSystem_PERS', '".mysql_real_escape_string($key)."', '".mysql_real_escape_string($value)."')";
			mysql_query($query, $connection) or die('Error: '.mysql_error());
			$res = $value;
		} else {
			$res = mysql_result($result,0);
		}
		return $res;
	}

	public function updateValue($connection, $prefix, $key, $value) {
		$query = "UPDATE ".$prefix."uddepf_config"." SET `value` = '".mysql_real_escape_string($value)."' WHERE plugin = 'PFSystem_PERS' AND varname = '".mysql_real_escape_string($key)."'";
		$result = mysql_query($query, $connection) or die('Error: '.mysql_error());
		return $result;
	}
}
