<?php
// ********************************************************************************************
// Title          udde Picture Frame Information System (uddePF)
// Description    Picture Frame Information System for Joomla 1.5 and Joomla 1.6
// Author          2011 Stephan Slabihoud
// License        This is free software and you may redistribute it under the GPL.
//                uddePF comes with absolutely no warranty.
//                Use at your own risk. For details, see the license at
//                http://www.gnu.org/licenses/gpl.txt
//                Other licenses can be found in LICENSES folder.
//                Redistributing this file is only allowed when keeping the header unchanged.
//
// It uses parts of
//
// InfoFrame (image generator for digital picture frames)
// Copyright (C) 2009  Tobias Kolb
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// ********************************************************************************************

if (!(defined('_JEXEC') || defined('_VALID_MOS'))) { die( 'Direct Access to this location is not allowed.' ); }

class PFWeatherPlugin extends PFPlugin
{
	protected $config_city = "Bottrop,Germany";
	protected $config_long = 51.5;
	protected $config_lat = 6.90;
	protected $config_iconset = "iconbest";
	
	protected $cachefile = NULL;

	public function __construct($dbconn, $tableprefix) {
        parent::__construct($dbconn, $tableprefix); 
		if (!file_exists( dirname(__FILE__).'/../cache/') )
			mkdir( dirname(__FILE__).'/../cache' );
		$this->cachefile = dirname(__FILE__).'/../cache/weather.xml';
	}
	
	public function getPluginName() {
		return _UDDEPFADM_WEATHER_PLUGIN;
	}
	
	public function getPluginVersion() {
		return "1.1";
	}

	public function showConfiguration() {
        parent::showConfigurationHelper(UDDEPF_MODE_ENABLE|UDDEPF_MODE_UPDATE|UDDEPF_MODE_PAGES); 
		uddePF::admText($this->config_city, 20, 'config_city', _UDDEPFADM_CITY_HEAD, _UDDEPFADM_CITY_EXP, "", true);
		uddePF::admText($this->config_long, 10, 'config_long', _UDDEPFADM_WLONG_HEAD, _UDDEPFADM_WLONG_EXP, "");
		uddePF::admText($this->config_lat, 10, 'config_lat', _UDDEPFADM_WLAT_HEAD, _UDDEPFADM_WLAT_EXP, "");
		$tdirs = Array();
		$dir = dirname(__FILE__).'/'.$this->plugin;		// no slash
		if ($hd = opendir($dir)) {
			while ($sz = readdir($hd)) { 
				if (!preg_match("/\./",$sz) && !preg_match("/language/",$sz))
					$tdirs[] = $sz;
			}
			closedir($hd);
		}
		asort($tdirs);
		$arr = Array();
		foreach($tdirs as $tdir) {
			$arr[ $tdir ] = $tdir;
		}
		uddePF::admSelect($this->config_iconset, 'config_iconset', $arr, _UDDEPFADM_ICONSET_HEAD, _UDDEPFADM_ICONSET_EXP);
	}

	public function getConfiguration() {
        parent::getConfigurationHelper(); 
		$this->config_city					= uddePF::getParameter( 'config_city', 'Bottrop,Germany', 'post', 'string' );
		$this->config_long					= (float)uddePF::getParameter( 'config_long', 51.5, 'post', 'float' );
		$this->config_lat					= (float)uddePF::getParameter( 'config_lat', 6.90, 'post', 'float' );
		$this->config_iconset				= uddePF::getParameter( 'config_iconset', 'iconbest', 'post', 'string' );
	}

	public function saveConfiguration() {
		$data = Array(
						"config_city"		=> $this->config_city,
						"config_long"		=> $this->config_long,
						"config_lat"		=> $this->config_lat,
						"config_iconset"	=> $this->config_iconset
					 );
		parent::saveConfigurationHelper($data);
	}


	public function doUpdate($forceUpdate) {
		if (!$this->config_enable)
			return;

		if (!$forceUpdate) {
			if (!$this->checkIfUpdateRequired())		// check if update period is exceeded
				return;
		}

		$this->updateDone();

		$city = urlencode($this->config_city);
		$curl = curl_init();
		$file = fopen( $this->cachefile, "w");
		if ($file) {
			curl_setopt($curl, CURLOPT_URL, utf8_encode("http://www.google.de/ig/api?weather=$city&oe=UTF8"));
			curl_setopt($curl, CURLOPT_FILE, $file);
			curl_exec($curl);
			curl_close($curl);
			fclose($file);
		}
	}

	public function doOutput(&$image, $style, $forceUpdate, &$yoffset) {
		if (!$this->config_enable)
			return;

		$filename = $this->cachefile;
		if ($forceUpdate || !file_exists($filename))
			$this->doUpdate(true);

		if(file_exists($filename) && (filesize($filename) > 0)) {
			$xml = simplexml_load_file($filename);
			if($xml) {
				$unit_system = strtolower($xml->weather->forecast_information->unit_system['data']);	// SI or US

				$current_condition = $xml->weather->current_conditions->condition['data'];
				$current_temp_c = $xml->weather->current_conditions->temp_c['data'];
				$current_humidity = $xml->weather->current_conditions->humidity['data'];
				$current_wind_condition = $xml->weather->current_conditions->wind_condition['data'];
				$night = !$this->_isDaylight();

				$current_icon = $this->_getLocalWeatherImage($xml->weather->current_conditions->icon['data'], $night);

			$wicon = ImageCreateFromPNG ( $current_icon );
			$iw = imagesx($wicon);
			$ih = imagesy($wicon);
			$ratio_orig = $iw/$ih;
			$iw = $ratio_orig*120;
			$ih = 120;
			ImageCopyResampled($image, $wicon, imagesx($image) - $iw - 15, 5, 0, 0, $iw, $ih, imagesx($wicon), imagesy($wicon));
			ImageDestroy($wicon);

				$opt = array(
					'width' => 280,
					'align' => ALIGN_RIGHT
				);
				$text = $current_temp_c."C";
				imagettftextboxopt($image, 24, 0, imagesx($image)-300, 105, $style['textcolor'], $style['font'], $text, $opt);

				$text = _UDDEPFADM_WEATHER_CURRENT."\n$current_humidity\n".
				str_replace(_UDDEPFADM_WEATHER_REPLACE1, '', $current_wind_condition);
				imagettftextboxopt($image, 15, 0, imagesx($image)-300, 145, $style['textcolor'], $style['font'], $text, $opt);

				// forecast for today and next 3 days
				for ($i = 0; $i <= 3; $i++) {

					$low = $xml->weather->forecast_conditions[$i]->low['data'];
					$high = $xml->weather->forecast_conditions[$i]->high['data'];
					$condition = $xml->weather->forecast_conditions[$i]->condition['data'];
					$icon = $this->_getLocalWeatherImage($xml->weather->forecast_conditions[$i]->icon['data'], false);

					$wicon = ImageCreateFromPNG( $icon );
					$iw = imagesx($wicon);
					$ih = imagesy($wicon);
					if ($iw>=90) {
						$iw /= 2;
						$ih /= 2;
					}
					$ratio_orig = $iw/$ih;
					$iw = $ratio_orig*60;		// imagesx($wicon)
					$ih = 60;					// imagesy($wicon)

					$offset = 225 + ($ih * $i);
					if ($offset > (imagesy($image)))
						break;

					ImageCopyResampled($image, $wicon, imagesx($image) - $iw - 7, $offset+5, 0, 0, $iw, $ih, imagesx($wicon), imagesy($wicon));
					ImageDestroy($wicon);

					$day = $xml->weather->forecast_conditions[$i]->day_of_week['data'];
					if ($i == 0)
						$day = _UDDEPFADM_WEATHER_TODAY;
					if ($i == 1)
						$day = _UDDEPFADM_WEATHER_TOMORROW;
					
					$opt = array(
						'width' => 150,
						'v_align' => VALIGN_TOP,
						'align' => ALIGN_RIGHT
					);
					$text = $day."\n".$high." | ".$low."\n".$condition;
					$arr = imagettfbbox ( 13, 0, $style['font'], $text );
					$add = 6;
					imagettftextboxopt($image, 13, 0, imagesx($image)-7 -10 - $iw - 150, $offset+$add, $style['textcolor'], $style['font'], $text, $opt);
				}
			}
		} 
	}
	
	private function _getLocalWeatherImage($googleWeatherImage, $night) {
		$localpath = dirname(__FILE__).'/'.$this->plugin.'/'.$this->config_iconset.'/';
		$localImagePath = str_replace('/ig/images/weather/', $localpath, $googleWeatherImage);
		$localImagePath = str_replace('.gif', '.png', $localImagePath);
		$localImagePathNight = str_replace('.png', '_night.png', $localImagePath);
		// if parameter $night is true and night image exist use it
		if ($night && file_exists($localImagePathNight))
			$localImagePath = $localImagePathNight;
		else if (!file_exists($localImagePath))
			// if daylight image doesn't exist display N/A image
			$localImagePath = $localpath . 'na.png';
		return $localImagePath;
	}

	private function _getSunriseAndSunset($date) {
		$gmtoffset = 1; // Normalzeit (Winterzeit)
		if (date("I", $date) == 1)
			$gmtoffset = 2;       // Sommerzeit

		// Zenith am Beispiel des Sonnenuntergangs:
		$zenith = 90 + (5/6); // offiziell (Sonne hinter dem Horizont verschwunden, aber immer noch hell)
		//$zenith = 96; // brgerlich (Sonne 6 unter dem Horizont, als halbwegs Dunkel empfunden)
		//$zenith = 102; // nautisch (fr See-Navigation, Horizont noch, Sterne schon sichtbar)
		//$zenith = 108; // astronomisch (absolute Dunkelheit)

		$longitude = $this->config_long;
		$latitude = $this->config_lat;
		$sunrise = date_sunrise($date, SUNFUNCS_RET_TIMESTAMP, $longitude, $latitude, $zenith, $gmtoffset);
		$sunset = date_sunset($date, SUNFUNCS_RET_TIMESTAMP, $longitude, $latitude, $zenith, $gmtoffset);
		return array($sunrise, $sunset);
	}

	private function _isDaylight() {
		$timestamp = time();
		$suntimes = $this->_getSunriseAndSunset($timestamp);
		if (($timestamp > $suntimes[0]) && ($timestamp < $suntimes[1]))
			return true;
		else
			return false;
	}
}
